declare
  counter              numeric(16);

  -- report
  r_id                 numeric(16);
  r_name               varchar(256) := 'Certification - Fulfilment status summary';
  r_desc               varchar(512) := 'Status summary with details around the fulfilment status (applicable for Applications not subject to write back sync).';
  r_code               varchar(256) := null;
  r_i18n_mess          varchar(256) := null;
  r_dashboard          numeric(1) := 0; -- 0: report, 1: dashboard

  -- localization
  r_loc_id             numeric(16);
  r_loc_code           varchar(256);

  -- query
  rq_id                numeric(16);
  rq_name              varchar(256) := 'Certification - Fulfilment status report';
  rq_desc              varchar(512) := 'Status summary with details around the fulfilment status, (applicable for the application not subject to write back sync)';
  rqd                  clob;

  rv_name              varchar(256);
  repv_vis_type        numeric(2);

  rqf_type_def         varchar(256) := 'java.lang.String';
  rqf_width_def        numeric(16) := 300;

  rqf_name_old         varchar(256);
  rqf_name             varchar(256);
  rqf_type             varchar(256);
  rqf_width            numeric(16);
  rqf_ord_by           numeric(1);
  rqf_ord_by_clau      varchar(256);
  loc_code_old         varchar(256);
  loc_code             varchar(256);
  loc_rename           boolean;
  i18n_msg             varchar(256);
  repf_pos_id          numeric(16);
  repf_pos_id_old      numeric(16);

  repfr_id             numeric(16);
  repfr_name_old       varchar(256);
  repfr_name           varchar(256);
  repfr_value          varchar(256);
  repfr_l_id           numeric(16);
  repfr_l_code_old     varchar(256);
  repfr_l_code         varchar(256);
  repfr_i18n_msg       varchar(256);
  repfr_vis            numeric(16);
  repfr_mand           numeric(16);
  repfr_type           numeric(16);
  repfr_cust_id        numeric(16);
  repfr_cust_name      varchar(256);
  repfr_posit_id       numeric(16);
  repfr_posit_id_old   numeric(16);

  v_ret                numeric(16);

begin
  rqd := 'select distinct t.CAMPAIGN_NAME as CAMPAIGN_NAME,
'||'        t.CAMPAIGN_START as CAMPAIGN_START,
'||'        t.CAMPAIGN_END as CAMPAIGN_END,
'||'        t.CAMPAIGN_STATUS as CAMPAIGN_STATUS,
'||'        t.APPLICATION_NAME as APPLICATION_NAME,
'||'        t.ACCOUNT_CODE as ACCOUNT_CODE,
'||'        t.ACCOUNT_IS_DEFAULT as ACCOUNT_IS_DEFAULT,
'||'        t.ACCOUNT_CONFIGURATION as ACCOUNT_CONFIGURATION,
'||'        t.PERMISSION_NAME as PERMISSION_NAME,
'||'        t.USER_CODE as USER_CODE,
'||'        t.USER_NAME as USER_NAME,
'||'        t.USER_SURNAME as USER_SURNAME,
'||'        t.OU_CODE as OU_CODE,
'||'        t.OU_NAME as OU_NAME,
'||'        t.NOTE as NOTE,
'||'        t.ACTION_NOTE as ACTION_NOTE,
'||'        t.ATTESTATION_STATUS as ATTESTATION_STATUS,
'||'        t.REVIEWED_BY_INFO REVIEWED_BY_INFO,
'||'        t.REVIEW_DATE as REVIEW_DATE,
'||'        t.REVOKE_FULFILLMENT as REVOKE_FULFILLMENT
'||'   from
'||'   (
'||'   select att.name as CAMPAIGN_NAME,
'||'          att.start_date as CAMPAIGN_START,
'||'          att.end_date as CAMPAIGN_END,
'||'          ''Active'' as CAMPAIGN_STATUS,
'||'          a.name as APPLICATION_NAME,
'||'          pwdm.code as ACCOUNT_CODE,
'||'          case
'||'            when pwdm.is_default = 1 then ''True'' 
'||'            when pwdm.is_default = 0 then ''False''
'||'            else ''Invalid''
'||'          end as ACCOUNT_IS_DEFAULT,
'||'          pwdcfg.name as ACCOUNT_CONFIGURATION,
'||'          e.name as PERMISSION_NAME,
'||'          p.code as USER_CODE,
'||'          p.name as USER_NAME,
'||'          p.surname as USER_SURNAME,
'||'          ou.code as OU_CODE,
'||'          ou.name as OU_NAME,
'||'          er.note as NOTE,
'||'          er.action_note as ACTION_NOTE,
'||'          case
'||'            when er.review_state = 0 then ''Not recertified yet''
'||'            when er.review_state = 1 then ''Approved''
'||'            when er.review_state = 2 or er.review_state = 3 then ''Revoked''
'||'            when er.review_state >= 10 then ''Other''
'||'          end as ATTESTATION_STATUS,
'||'          er.reviewed_by_info as REVIEWED_BY_INFO,
'||'          er.review_date as REVIEW_DATE,
'||'          case
'||'            when er.review_state in ( 0, 1 ) then null
'||'            when er.employment is not null then ''Pending''
'||'            else ''Removed''
'||'          end as REVOKE_FULFILLMENT,
'||'          att.state as campaign_status_num,
'||'          ou.id as ou_id,
'||'          a.id as app_id
'||'     from #pmschema#.attestation              att,
'||'          #pmschema#.employment_review        er
'||'           left join #pmschema#.employment_pwdmanagement epwdm on epwdm.employment = er.employment
'||'           left join #pmschema#.pwdmanagement pwdm on epwdm.pwdmanagement = pwdm.id
'||'           left join #pmschema#.pwdcfg pwdcfg on pwdm.pwdcfg = pwdcfg.id
'||'           left join #pmschema#.target tar on pwdm.pwdcfg = tar.pwdcfg
'||'           left join #pmschema#.application a on a.target = tar.id,
'||'          #pmschema#.person                   p,
'||'          #pmschema#.organizational_unit      ou,
'||'          #pmschema#.entitlement              e,
'||'          #pmschema#.entitlement_flat_hier  efh
'||'    where att.id = er.attestation
'||'      and att.type = 1
'||'      and er.person = p.id
'||'      and p.organizational_unit = ou.id
'||'      and er.entitlement = e.id
'||'      and e.id = efh.parent
'||'      and efh.child_application = a.id
'||'      and efh.child_int_type = 1
'||'      and ou.hierarchy = 1
'||'      and not exists
'||'       (select 1 from #pmschema#.employment_review_h erh
'||'        where erh.entitlement_name=e.name
'||'        and erh.application_name=a.name
'||'        and erh.attestation_name=att.name
'||'        and erh.person=er.person)
'||'   union
'||'   select erh.attestation_name as CAMPAIGN_NAME,
'||'          null as CAMPAIGN_START,
'||'          null as CAMPAIGN_END,
'||'          ''Inactive'' as CAMPAIGN_STATUS,
'||'          a.name as APPLICATION_NAME,
'||'          case
'||'            when erph.pwdmanagement_code IS NOT NULL then erph.pwdmanagement_code
'||'            else ''N/A''
'||'          end as ACCOUNT_CODE,
'||'          case
'||'            when erph.is_default = 1 then ''True''  
'||'            when erph.is_default = 0 then ''False''
'||'            else ''N/A''
'||'          end as ACCOUNT_IS_DEFAULT,
'||'          case
'||'            when erph.pwdcfg_name IS NOT NULL then erph.pwdcfg_name
'||'            else ''N/A''
'||'          end as ACCOUNT_CONFIGURATION,
'||'          e.name as PERMISSION_NAME,
'||'          p.code as USER_CODE,
'||'          p.name as USER_NAME,
'||'          p.surname as USER_SURNAME,
'||'          ou.code as OU_CODE,
'||'          ou.name as OU_NAME,
'||'          erh.note as NOTE,
'||'          erh.action_note as ACTION_NOTE,
'||'          case
'||'            when erh.review_state = 0 then ''Not recertified''
'||'            when erh.review_state = 1 then ''Approved''
'||'            when erh.review_state = 2 or erh.review_state = 3 then ''Revoked''
'||'            when erh.review_state >= 10 then ''Other''
'||'          end as ATTESTATION_STATUS,
'||'          erh.cert_first_owner_name || '' '' || erh.cert_first_owner_surname || '' ('' || erh.cert_first_owner_code || '')'' as REVIEWED_BY_INFO,
'||'          erh.review_date as REVIEW_DATE,
'||'          case
'||'            when erh.review_state in ( 0, 1 ) then null
'||'            else ''Removed''
'||'          end as REVOKE_FULFILLMENT,
'||'          null as campaign_status_num,
'||'          ou.id as ou_id,
'||'          a.id as app_id
'||'     from #pmschema#.employment_review_h       erh left join #pmschema#.employment_review_pwdman_h erph ON erph.employment_review_h = erh.id,
'||'          #pmschema#.person                    p,
'||'          #pmschema#.organizational_unit       ou,
'||'          #pmschema#.entitlement               e,
'||'          #pmschema#.entitlement_flat_hier     efh,
'||'          #pmschema#.application               a,
'||'          #pmschema#.target                    tar,
'||'          #pmschema#.pwdcfg                    pwdcfg,
'||'		  #pmschema#.pwdmanagement             pwdman
'||'    where erh.person = p.id
'||'      and p.organizational_unit = ou.id
'||'      and erh.entitlement = e.id
'||'      and e.id = efh.parent
'||'      and efh.child_application = a.id
'||'      and efh.child_int_type = 1
'||'      and ou.hierarchy = 1
'||'      and p.id = pwdman.person
'||'      and pwdman.pwdcfg = pwdcfg.id
'||'   	  and pwdman.pwdcfg = tar.pwdcfg
'||'      and pwdcfg.id = tar.pwdcfg
'||'      and tar.id = a.TARGET
'||'      and p.id = erh.PERSON
'||'   union
'||'   select erh.attestation_name as CAMPAIGN_NAME,
'||'          null as CAMPAIGN_START,
'||'          null as CAMPAIGN_END,
'||'          ''Inactive'' as CAMPAIGN_STATUS,
'||'          erh.application_name as APPLICATION_NAME,
'||'          erph.pwdmanagement_code as ACCOUNT_CODE,
'||'          case
'||'            when erph.is_default = 1 then ''True'' 
'||'            when erph.is_default = 0 then ''False''
'||'            else ''Invalid''
'||'          end as ACCOUNT_IS_DEFAULT,
'||'          erph.pwdcfg_name as ACCOUNT_CONFIGURATION,
'||'          erh.entitlement_name as PERMISSION_NAME,
'||'          p.code as USER_CODE,
'||'          p.name as USER_NAME,
'||'          p.surname as USER_SURNAME,
'||'          ou.code as OU_CODE,
'||'          ou.name as OU_NAME,
'||'          erh.note as NOTE,
'||'          erh.action_note as ACTION_NOTE,
'||'          case
'||'            when erh.review_state = 0 then ''Not recertified''
'||'            when erh.review_state = 1 then ''Approved''
'||'            when erh.review_state = 2 or erh.review_state = 3 then ''Revoked''
'||'            when erh.review_state >= 10 then ''Other''
'||'          end as ATTESTATION_STATUS,
'||'          null as REVIEWED_BY_INFO,
'||'          erh.review_date as REVIEW_DATE,
'||'          ''System deleted'' as REVOKE_FULFILLMENT,
'||'          null as campaign_status_num,
'||'          ou.id as ou_id,
'||'          null as app_id
'||'     from #pmschema#.employment_review_h       erh left join #pmschema#.employment_review_pwdman_h erph ON erph.employment_review_h = erh.id,
'||'          #pmschema#.person                    p,
'||'          #pmschema#.organizational_unit       ou,
'||'          #pmschema#.application               a,
'||'          #pmschema#.target                    tar,
'||'          #pmschema#.pwdcfg                    pwdcfg
'||'    where erh.person = p.id
'||'      and p.organizational_unit = ou.id
'||'      and erh.entitlement is null
'||'      and ou.hierarchy = 1
'||'      and a.target = tar.id
'||'      and pwdcfg.id = tar.pwdcfg
'||'      and pwdcfg.name = erph.pwdcfg_name
'||' ) t,
'||'   #schema_tmp#.tmp_rep_application tmp1,
'||'   #schema_tmp#.tmp_rep_organizational_unit tmp2
'||' where
'||'   lower(t.attestation_status) = lower(''#attestation_status#'')
'||'   and lower(t.revoke_fulfillment) = lower(''#revoke_fulfillment#'')
'||'   and (tmp1.id = t.app_id or t.app_id is null)
'||'   and tmp2.id = t.ou_id
';

  select count(1) into counter from r_query t where lower ( t.name ) = lower ( rq_name ) and t.product = 1;
  if ( counter = 0 ) then
    select max(id) + 1 into rq_id from r_query t;

    insert into r_query ( id, name, description, product )
    values ( rq_id, rq_name, rq_desc, 1 );

    insert into r_query_data ( r_query, query )
    values ( rq_id, rqd );
  else
    select t.id into rq_id from r_query t where lower ( t.name ) = lower ( rq_name ) and t.product = 1;

    update r_query set description = rq_desc where id = rq_id;

    update r_query_data set query = rqd where r_query = rq_id;
  end if;

  -- Report FIELD 1
  rqf_name_old    := null;
  rqf_name        := 'ACCOUNT_CODE';
  rqf_type        := rqf_type_def;
  rqf_width       := rqf_width_def;
  rqf_ord_by      := 0;
  rqf_ord_by_clau := null;
  loc_code_old    := null;
  loc_code        := 'account.code';
  loc_rename      := true;
  i18n_msg        := 'Account Code';
  repf_pos_id_old := null;
  repf_pos_id     := 6;

  v_ret := inst_rep_add_col ( rq_id, rqf_name_old, rqf_name, rqf_type, rqf_width, rqf_ord_by, rqf_ord_by_clau,
                              loc_code_old, loc_code, loc_rename, i18n_msg, repf_pos_id_old, repf_pos_id );
                              
    -- Report FIELD 2
  rqf_name_old    := null;
  rqf_name        := 'ACCOUNT_IS_DEFAULT';
  rqf_type        := rqf_type_def;
  rqf_width       := rqf_width_def;
  rqf_ord_by      := 0;
  rqf_ord_by_clau := null;
  loc_code_old    := null;
  loc_code        := 'account.isdefault';
  loc_rename      := true;
  i18n_msg        := 'Is Default';
  repf_pos_id_old := null;
  repf_pos_id     := 7;

  v_ret := inst_rep_add_col ( rq_id, rqf_name_old, rqf_name, rqf_type, rqf_width, rqf_ord_by, rqf_ord_by_clau,
                              loc_code_old, loc_code, loc_rename, i18n_msg, repf_pos_id_old, repf_pos_id );

                                    
  -- Report FIELD 3
  rqf_name_old    := null;
  rqf_name        := 'ACCOUNT_CONFIGURATION';
  rqf_type        := rqf_type_def;
  rqf_width       := rqf_width_def;
  rqf_ord_by      := 0;
  rqf_ord_by_clau := null;
  loc_code_old    := null;
  loc_code        := 'account.configuration';
  loc_rename      := true;
  i18n_msg        := 'Account Configuration';
  repf_pos_id_old := null;
  repf_pos_id     := 8;

  v_ret := inst_rep_add_col ( rq_id, rqf_name_old, rqf_name, rqf_type, rqf_width, rqf_ord_by, rqf_ord_by_clau,
                              loc_code_old, loc_code, loc_rename, i18n_msg, repf_pos_id_old, repf_pos_id );
                              
                              
  -- Report FIELD 4
  rqf_name_old    := null;
  rqf_name        := 'NOTE';
  rqf_type        := rqf_type_def;
  rqf_width       := rqf_width_def;
  rqf_ord_by      := 0;
  rqf_ord_by_clau := null;
  loc_code_old    := null;
  loc_code        := 'note';
  loc_rename      := true;
  i18n_msg        := 'Note';
  repf_pos_id_old := null;
  repf_pos_id     := 10;

  v_ret := inst_rep_add_col ( rq_id, rqf_name_old, rqf_name, rqf_type, rqf_width, rqf_ord_by, rqf_ord_by_clau,
                              loc_code_old, loc_code, loc_rename, i18n_msg, repf_pos_id_old, repf_pos_id );
                              
   -- Report FIELD 4
  rqf_name_old    := null;
  rqf_name        := 'ACTION_NOTE';
  rqf_type        := rqf_type_def;
  rqf_width       := rqf_width_def;
  rqf_ord_by      := 0;
  rqf_ord_by_clau := null;
  loc_code_old    := null;
  loc_code        := 'action.note';
  loc_rename      := true;
  i18n_msg        := 'Action Note';
  repf_pos_id_old := null;
  repf_pos_id     := 11;

  v_ret := inst_rep_add_col ( rq_id, rqf_name_old, rqf_name, rqf_type, rqf_width, rqf_ord_by, rqf_ord_by_clau,
                              loc_code_old, loc_code, loc_rename, i18n_msg, repf_pos_id_old, repf_pos_id );


end;
/
commit;

